//+------------------------------------------------------------------+
//|                                                       Signal.mq4 |
//+------------------------------------------------------------------+

#property indicator_chart_window

#include <hanover --- function header (np).mqh>

extern int     scaleX               = 16,                                // horizontal interval at which the symbols are created
               scaleY               = 10,                                // vertical interval
               offsetX              = 35,                                // horizontal indent for first symbol
               offsetY              = 50,                                // vertical indent
               fontSize             = 9,                                 // font size
               corner               = 1,                                 // corner: 0=top left; 1=top right; 2=bottom left; 3=bottom right
               symbolCodeBuy        = 217,                               // symbol code for a buy signal
               symbolCodeSell       = 218,                               // sell signal
               symbolCodeNoSignal   = 167;                               // no signal               
extern string  symbolFont           = "Wingdings";                       // font for text
extern color   signalBBColor        = RoyalBlue,                         // color of the symbol for a STRONG buy
               signalBuyColor       = Green,                             // color of the symbol of a weak buy
               noSignalColor        = Yellow,                            // no signal
               signalSellColor      = DarkGoldenrod,                     // color for a weak sell signal
               signalSSColor        = Red,                               // color for a STRONG sell
               textColor            = White;                             // color of all text
extern string  MAperiods            = "5,10,15,20,25,30,40,50,75,100",   // periods for moving averages
               WPRvalues            = "14,-70,-30",                      // values for Williams %R (period,high,low)
               RSIvalues            = "14,70,30",                        // values for RSI  (period,high,low)          
               STOvalues            = "8,3,3,70,30";                     // values for stochastic (%K,%D,slowing,high,low)             
extern int     ADRdays              = 30;                                // days lookback for ADR calc (excludes Saturdays/Sundays)  
extern string  RefreshPeriod        = "T";                               // 'T' = refresh values every new tick; otherwise use M1, M5, etc
extern bool    ShowMAlights         = true,                              // turn MA 'lights' on/off
               ShowOSClights        = true,                              // turn oscillator 'lights' on/off
               ShowSARlights        = true,                              // turn Parabolic SAR 'lights' on/off
               ShowPrice            = true,                              // display Price and Ticker
               ShowSpread           = true,                              // display spread ('S') data row 
               ShowPipValue         = true,                              // display pip value ('$') data row 
               ShowADR              = true,                              // display daily ranges ('R') data row 
               ShowSwapRates        = true,                              // display carry/swap/rollover ('C') data row 
               ShowCandleTimes      = true,                              // display candle times ('T') data row 
               UpdateTicker         = false;                             // activate Ticker yes/no
                
int      period[]           = {1,5,15,30,60,240,1440,10080,43200};
string   periodString[]     = {"1","5","15","30","H","4","D","W","M"};
string   signalNameString[] = {"MA","Osc","SAR"};
string   IndiName           = "#signal";
string   ccy, ticker;
int      tmf, dig, bStyle, bWidth, MAP[10], WPRV[3], RSIV[3], STOV[5], RefreshEveryXMins;
double   pnt, askp, bidp, lastbid, spr, tickval, lswap, sswap;
datetime prev_time;

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+
  ccy     = Symbol();
  tmf     = Period();
  bidp    = MarketInfo(ccy,MODE_BID);
  askp    = MarketInfo(ccy,MODE_ASK);
  pnt     = MarketInfo(ccy,MODE_POINT);
  dig     = MarketInfo(ccy,MODE_DIGITS);
  spr     = MarketInfo(ccy,MODE_SPREAD);
  tickval = MarketInfo(ccy,MODE_TICKVALUE);
  if (dig == 3 || dig == 5) {
    pnt     *= 10;
    spr     /= 10;
    tickval *= 10;
  }  

  RefreshEveryXMins = StrToTF(RefreshPeriod);

  StrToIntegerArray(MAperiods,MAP);
  StrToIntegerArray(WPRvalues,WPRV);
  StrToIntegerArray(RSIvalues,RSIV);
  StrToIntegerArray(STOvalues,STOV);
  
  prev_time  = -9999;
  del_obj();

  int y0=35;
  int y1=1;
  for (int x=0; x<9; x++)   {
    y0=0;
    for (int y=0; y<3; y++)   {
      if (y==0 && !ShowMAlights)    continue;
      if (y==1 && !ShowOSClights)   continue;
      if (y==2 && !ShowSARlights)   continue;
      string objname = IndiName + NumberToStr(x,"'-XY-'Z2") + NumberToStr(y,"'-'Z2");
      ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
      ObjectSet(objname,OBJPROP_CORNER,corner);    
      ObjectSet(objname,OBJPROP_XDISTANCE,x*scaleX+offsetX);
      ObjectSet(objname,OBJPROP_YDISTANCE,y0*scaleY+offsetY);
      ObjectSetText(objname,CharToStr(symbolCodeNoSignal),fontSize,"Wingdings",noSignalColor);  
      y0++;
  } }
  if (ShowMAlights || ShowOSClights || ShowSARlights)  {
    for (x=0; x<9; x++)  {
      objname = IndiName + NumberToStr(x,"'-X-'Z2");
      ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
      ObjectSet(objname,OBJPROP_CORNER,corner);
      ObjectSet(objname,OBJPROP_XDISTANCE,x*scaleX+offsetX);
      ObjectSet(objname,OBJPROP_YDISTANCE,offsetY-10);
      ObjectSetText(objname,periodString[x],8,"Tahoma",textColor);  
  } }  
  y0=0;
  for (y=0; y<3; y++)   {
    if (y==0 && !ShowMAlights)    continue;
    if (y==1 && !ShowOSClights)   continue;
    if (y==2 && !ShowSARlights)   continue;
    objname = IndiName + NumberToStr(y,"'-Y-'Z2");
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-25);
    ObjectSet(objname,OBJPROP_YDISTANCE,y*scaleY+offsetY);
    ObjectSetText(objname,signalNameString[y],8,"Tahoma",textColor);
    y0++;
  }
  y0=y0*10+5;
  if (ShowPrice)   {
    objname = IndiName + "-A";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0);
    objname = IndiName + "-T";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0+31);
    y0+=35;
  }
  if (ShowSpread)   {
  objname = IndiName + "-B";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0+y1*scaleY);
    y1 += 2;
  }  
  if (ShowPipValue)   {
    objname = IndiName + "-C";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0+y1*scaleY);
    y1 += 2;
  }  
  if (ShowADR)   {
    objname = IndiName + "-D";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0+y1*scaleY);
    y1 += 2;
  }
  if (ShowSwapRates)   {
    objname = IndiName + "-E";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0+y1*scaleY);
    y1 += 2;
  }
  if (ShowCandleTimes)   {
    objname = IndiName + "-F";
    ObjectCreate(objname,OBJ_LABEL,0,0,0,0,0);
    ObjectSet(objname,OBJPROP_CORNER,corner);
    ObjectSet(objname,OBJPROP_XDISTANCE,offsetX-20);
    ObjectSet(objname,OBJPROP_YDISTANCE,offsetY+y0+y1*scaleY);
  }

  lastbid = bidp;  
  ticker = StringRepeat(CharToStr(149),25);
  return(0);
}

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
  del_obj();
  return(0);
}

//+------------------------------------------------------------------+
void del_obj()  {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  if (RefreshEveryXMins < 0)
    return(0);
  if (RefreshEveryXMins == 0) {
    upd_obj();    
  }
  else {
    if (prev_time != iTime(ccy,RefreshEveryXMins,0))  {
      upd_obj();
      prev_time = iTime(ccy,RefreshEveryXMins,0);
  } }      
  return(0);
}

//+------------------------------------------------------------------+
void upd_obj()  {
//+------------------------------------------------------------------+
  ccy     = Symbol();
  tmf     = Period();
  bidp    = MarketInfo(ccy,MODE_BID);
  askp    = MarketInfo(ccy,MODE_ASK);
  pnt     = MarketInfo(ccy,MODE_POINT);
  dig     = MarketInfo(ccy,MODE_DIGITS);
  spr     = MarketInfo(ccy,MODE_SPREAD);
  tickval = MarketInfo(ccy,MODE_TICKVALUE);
  lswap   = MarketInfo(ccy,MODE_SWAPLONG);
  sswap   = MarketInfo(ccy,MODE_SWAPSHORT);
  if (dig == 3 || dig == 5) {
    pnt     *= 10;
    spr     /= 10;
    tickval *= 10;
  }  

  if (ShowMAlights)   {
    for (int x=0; x<9; x++)   {
      int y=0;
      string objname = IndiName + NumberToStr(x,"'-XY-'Z2") + NumberToStr(y,"'-'Z2");
      int c=0;
      for (int i=0; i<10; i++)  {
  //      if (iClose(Symbol(),period[x],0) > iMA(Symbol(),period[x],MAP[i],0,0,0,0))      c++;    else
  //      if (iClose(Symbol(),period[x],0) < iMA(Symbol(),period[x],MAP[i],0,0,0,0))      c--;
        if (iMA(Symbol(),period[x],MAP[i],0,0,0,0) > iMA(Symbol(),period[x],MAP[i],0,0,0,10))      c++;    else
        if (iMA(Symbol(),period[x],MAP[i],0,0,0,0) < iMA(Symbol(),period[x],MAP[i],0,0,0,10))      c--;
      }
      if (c>8)   ObjectSetText(objname,CharToStr(symbolCodeBuy),fontSize,symbolFont,signalBBColor);     else
      if (c>4)   ObjectSetText(objname,CharToStr(symbolCodeBuy),fontSize,symbolFont,signalBuyColor);    else
      if (c<-8)  ObjectSetText(objname,CharToStr(symbolCodeSell),fontSize,symbolFont,signalSSColor);    else
      if (c<-4)  ObjectSetText(objname,CharToStr(symbolCodeSell),fontSize,symbolFont,signalSellColor);  else
                 ObjectSetText(objname,CharToStr(symbolCodeNoSignal),fontSize,symbolFont,noSignalColor);   
  } }

  if (ShowOSClights)   {
    for (x=0; x<9; x++)   {
      y=1;
      objname = IndiName + NumberToStr(x,"'-XY-'Z2") + NumberToStr(y,"'-'Z2");
      c=0;    
      if (MathAbs(iWPR(Symbol(),period[x],WPRV[0],0)) < -WPRV[2])   c++;    else
      if (MathAbs(iWPR(Symbol(),period[x],WPRV[0],0)) > -WPRV[1])   c--;
      if (iRSI(Symbol(),period[x],RSIV[0],0,0) > RSIV[1])         c++;    else
      if (iRSI(Symbol(),period[x],RSIV[0],0,0) < RSIV[2])         c--;
      if (iStochastic(Symbol(),period[x],STOV[0],STOV[1],STOV[2],0,0,0,0) > STOV[3])   c++;    else
      if (iStochastic(Symbol(),period[x],STOV[0],STOV[1],STOV[2],0,0,0,0) < STOV[4])   c--;
    
      if (c>2)   ObjectSetText(objname,CharToStr(symbolCodeBuy),fontSize,symbolFont,signalBBColor);     else
      if (c>1)   ObjectSetText(objname,CharToStr(symbolCodeBuy),fontSize,symbolFont,signalBuyColor);    else
      if (c<-2)  ObjectSetText(objname,CharToStr(symbolCodeSell),fontSize,symbolFont,signalSSColor);    else
      if (c<-1)  ObjectSetText(objname,CharToStr(symbolCodeSell),fontSize,symbolFont,signalSellColor);  else
                 ObjectSetText(objname,CharToStr(symbolCodeNoSignal),fontSize,symbolFont,noSignalColor);   
  } }

  if (ShowSARlights)   {
    for (x=0; x<9; x++)   {
      y=2;
      objname = IndiName + NumberToStr(x,"'-XY-'Z2") + NumberToStr(y,"'-'Z2");
      if (iSAR(Symbol(),period[x],0.02,0.2,0) < Close[0])
        ObjectSetText(objname,CharToStr(symbolCodeBuy),fontSize,symbolFont,signalBBColor);
      else
        ObjectSetText(objname,CharToStr(symbolCodeSell),fontSize,symbolFont,signalSSColor);
  } }

  double sum = 0;
  double cnt = 0;
  double hi  = 0;
  double lo  = 999999;
  for (int k=1; k<=ADRdays; k++)   {
    datetime dt = iTime(ccy,PERIOD_D1,k);
    if (TimeDayOfWeek(dt) == 0)   continue;
    double range = (iHigh(ccy,PERIOD_D1,k) - iLow(ccy,PERIOD_D1,k))/pnt;
    sum += range;
    cnt += 1;
    hi   = MathMax(hi,range);
    lo   = MathMin(lo,range);
  }
  double fropen = (iClose(ccy,PERIOD_D1,0) - iOpen(ccy,PERIOD_D1,0))/pnt;
  double tdy    = (iHigh(ccy,PERIOD_D1,0) - iLow(ccy,PERIOD_D1,0))/pnt;
  double adr    = DivZero(sum,cnt);
  double pctadr = DivZero(100*tdy*cnt,sum);
  double pctspr = DivZero(100*spr*cnt,sum);

  if (ShowPrice)   {
    objname = IndiName + "-A";
    if (dig>3)  
      string pmsk = "R2.4";
    else
      pmsk = "R4.2";
    string objtext = NumberToStr(bidp,pmsk+"'   '");

    objtext = StringSubstr(objtext,0,7); // + "(" + StringSubstr(objtext,7,1) + ")";
    color colr = textColor;
    if (bidp > lastbid)   colr = Green;
    else
    if (bidp < lastbid)   colr = Red;
    ObjectSetText(objname,objtext,18,"Arial Black",colr);  
    objname = IndiName + "-T";
    if (UpdateTicker)
      if (bidp > lastbid)   ticker = StringSubstr(ticker,1) + CharToStr(188);   else
      if (bidp < lastbid)   ticker = StringSubstr(ticker,1) + CharToStr(189);   else
                            ticker = StringSubstr(ticker,1) + CharToStr(151);
    ObjectSetText(objname,ticker,8,"Wingdings 2",textColor);  
  }
  if (ShowSpread)   {
    objname = IndiName + "-B";
    if (corner==0 || corner==2)
      objtext = NumberToStr(spr*tickval,"'S  $'TR4.2") + NumberToStr(spr,"'    'TR4.1") + NumberToStr(pctspr,"'  ('TR3.1'%)'");
    else
      objtext = NumberToStr(spr*tickval,"'$'TR4.2") + NumberToStr(spr,"'    'TR4.1") + NumberToStr(pctspr,"'  ('TR3.1'%)  S'");
    ObjectSetText(objname,objtext,fontSize,"Verdana",textColor);  
  }
  if (ShowPipValue)   {
    objname = IndiName + "-C";
    if (corner==0 || corner==2)
      objtext = NumberToStr(tickval,"'$  'TR4.2") + NumberToStr(fropen,"'       'TR+3");
    else  
      objtext = NumberToStr(fropen,"TR+3'       '") + NumberToStr(tickval,"TR4.2'  $'");
    ObjectSetText(objname,objtext,fontSize,"Verdana",textColor);  
  }
  if (ShowADR)   {
    objname = IndiName + "-D";
  //  if (corner==0 || corner==2)
  //    objtext = NumberToStr(tdy,"'R  'TR4") + NumberToStr(adr,"' / 'TR4") + NumberToStr(pctadr,"'  ('TR3'%)'");
  //  else  
  //    objtext = NumberToStr(tdy,"TR4") + NumberToStr(adr,"' / 'TR4") + NumberToStr(pctadr,"'  ('TR3'%)  R'");
    if (corner==0 || corner==2)
      objtext = NumberToStr(tdy,"'R  'TR4") + NumberToStr(adr,"' / 'TR4") + NumberToStr(lo,"'  ['TR4','") + NumberToStr(hi,"TR4']'") + NumberToStr(pctadr,"'  ('TR3'%)'");
    else  
      objtext = NumberToStr(tdy,"TR4") + NumberToStr(adr,"' / 'TR4")  + NumberToStr(lo,"'  ['TR4','") + NumberToStr(hi,"TR4']'") + NumberToStr(pctadr,"'  ('TR3'%)  R'");
    ObjectSetText(objname,objtext,fontSize,"Verdana",textColor);  
  }
  if (ShowSwapRates)   {
    objname = IndiName + "-E";
    if (corner==0 || corner==2)
      objtext = NumberToStr(lswap,"'C  'TR+2.3") + NumberToStr(sswap,"'   'TR+2.3") + NumberToStr(1/bidp,"'    '"+pmsk);
    else  
      objtext = NumberToStr(1/bidp,pmsk+"'    '") + NumberToStr(lswap,"TR+2.3") + NumberToStr(sswap,"'   'TR+2.3'  C'");
    ObjectSetText(objname,objtext,fontSize,"Verdana",textColor);  
  }
  if (ShowCandleTimes)   {
    datetime tsvr = TimeCurrent();
    datetime tloc = TimeLocal();
    datetime tcan = Time[0] + 60*Period() - tsvr;
    double   tpct = 100 - 100*tcan/(60*Period());
    string   tmsk = "   I:S";
    if (Period() > 60)
      tmsk = "   H!hI!m";
    objname = IndiName + "-F";
  //  if (corner==0 || corner==2)
  //    objtext = DateToStr(tsvr,"'T  'H:I") + DateToStr(tloc,"   H:I") + DateToStr(tcan,tmsk);
  //  else  
  //    objtext = DateToStr(tsvr,"H:I") + DateToStr(tloc,"   H:I") + DateToStr(tcan,tmsk+"'  T'");
    if (corner==0 || corner==2)
      objtext = DateToStr(tsvr,"'T  'H:I:S") + NumberToStr(-tpct,"(6%") + DateToStr(tcan,tmsk);
    else  
      objtext = DateToStr(tsvr,"H:I:S") + NumberToStr(-tpct,"(6%") + DateToStr(tcan,tmsk+"'  T'");
    ObjectSetText(objname,objtext,fontSize,"Verdana",textColor);  
  }

  lastbid = bidp;
  return(0);
}

//+------------------------------------------------------------------+
#include <hanover --- extensible functions (np).mqh>

